window.indexedDB = window.indexedDB ||
	window.mozIndexedDB ||
	window.webkitIndexedDB ||
	window.msIndexedDB;

window.IDBTransaction = window.IDBTransaction ||
	window.webkitIDBTransaction ||
	window.msIDBTransaction;

window.IDBKeyRange = window.IDBKeyRange ||
	window.webkitIDBKeyRange ||
	window.msIDBKeyRange;

(function($) {
	'use strict';
	var DB_NAME = 'joyshebao';
	var DB_VERSION = 8;
	var READ_WRITE = 'readwrite';
	var READ_ONLY = 'readonly';
	var stores = {
		'adGames':{keyName:'posId'},
		'SecInfo':{keyName:'id'},
		'SecList':{keyName:'id'},
		'FunInfo':{keyName:'id'},
		'cityList':{keyName:'id'},
		'joyState':{keyName:'id'},
		'joyData':{keyName:'id'},
		'joyImgs':{keyName:'id'},
		'InsIAInfo':{keyName:'id'},
		'InsPayState':{keyName:'id'},
		'InsPayedMonths':{keyName:'id'},
		'retireInfoState':{keyName:'securityAcctId'},
		'retPenTrial':{keyName:'securityAcctId'},
		'medical':{keyName:'id'},
		'hospitalList':{keyName:'id'},
		'news':{keyName:'cateId'},
		'feedList':{keyName:'tabId'},
		'reimburse':{keyName:'cityType'}
	};
	var output = {
		debug:function(args){
			console.debug(args);
		},
		error:function(args){
			console.error(args);
		}
	};
		
	var _db = null;
	(function(){
		var sct = Date.now();
		var request = indexedDB.open(DB_NAME, DB_VERSION);
		request.onupgradeneeded = function(event){
			var db = event.target.result;
			console.log('update IndexDB=>joyshebao');
			console.log('currStores:',db.version,db.objectStoreNames);
			console.log('reqStores:',DB_VERSION,stores);
			for(var s in stores){
				if(!db.objectStoreNames.contains(s)){
					db.createObjectStore(s, {keyPath:stores[s].keyName});
				}
			}
			console.log('update IndexDB,Cost:',Date.now()-sct);
		};
		request.onerror = function(event) {
			var err = event.target.error;
			output.error(err.name+':['+err.code+']'+err.message);
		};
	})();

	function _regErrorHandler(request,deferred){
		request.onerror = function(event) {
			var err = event.target.error;
			output.error(err.name+':['+err.code+']'+err.message);
			if(deferred) deferred.reject(err);
		};
	};
	function _getDbObj(callback,deferred){
		var request = indexedDB.open(DB_NAME);
		_regErrorHandler(request,deferred);
		request.onsuccess = function(event) {
			callback(request.result);
		};
	};
	function _putDataToDb(db,storeName,data,deferred){
		var txn, store,req,res;
		txn = db.transaction(storeName, READ_WRITE);
		store = txn.objectStore(storeName);
		req = store.put(data);
		txn.oncomplete = function() {
			deferred.resolve(res);
		};
		req.onsuccess = function(event) {
			res = event.target.result;
		};
		_regErrorHandler(req,deferred);
	};
 	function _putDataToStore(store,data,deferred){
		var res,
			txn = store.transaction,
	 		req = store.put(data);
		txn.oncomplete = function() {
			deferred.resolve(res);
		};
		req.onsuccess = function(event) {
			res = event.target.result;
		};
		_regErrorHandler(req,deferred);
	};
	function _createStore(db,storeName,keyOptions,index){
		var store = db.createObjectStore(storeName, keyOptions); //创建对象库
		for(var i in index) { //创建索引
			store.createIndex(
				index[i].indexName,
				index[i].indexKey,
				index[i].indexOptions);
		}
		return store;
	};

	function getDB(){
		var deferred = Q.defer();
		_getDbObj(function(db){
			deferred.resolve(db);
		},deferred);
		return deferred.promise;
	}
	function delDB(){
		indexedDB.deleteDatabase(DB_NAME);
	}
	function getStoreNames(){
		var deferred = Q.defer();
		_getDbObj(function(db){
			 deferred.resolve(db.objectStoreNames);
		},deferred);
		return deferred.promise;
	}

	var db = Object.freeze({
		getDB: getDB,
		delDB: delDB,
		getStoreNames: getStoreNames
	});
	 

	function createStore(name,keyOptions,index){
		var deferred = Q.defer();
		var request = indexedDB.open(DB_NAME, Date.now());
		_regErrorHandler(request,deferred);
		request.onupgradeneeded = function(event){
			var db = event.target.result,
				store = _createStore(db,name,keyOptions,index);
			deferred.resolve(store);
		};
		return deferred.promise;
	}
	function getStore(name,mode){
		var deferred = Q.defer();
		_getDbObj(function(db){
			if(!db.objectStoreNames.contains(name)){
				deferred.resolve(null);
			}else{
				var txn = db.transaction(name, mode || READ_ONLY),
					store = txn.objectStore(name);	
				deferred.resolve(store);
			}
		},deferred);
		return deferred.promise;
	}
	function deleteStore(name){
		var deferred = Q.defer();
		var request = indexedDB.open(DB_NAME, Date.now());
		_regErrorHandler(request,deferred);
		request.onupgradeneeded = function(event) {
			var db = event.target.result;
			_regErrorHandler(db,deferred);
			if(!db.objectStoreNames.contains(name)){
				deferred.resolve(0);
			}else{
				db.deleteObjectStore(name);
				deferred.resolve(1);
			}
		};
		return deferred.promise;
	}
	
	var store = Object.freeze({
		createStore: createStore,
		getStore: getStore,
		deleteStore: deleteStore
	});
	
	
	function getAllData(storeName){
		var deferred = Q.defer();
		_getDbObj(function(db){
			if(!db.objectStoreNames.contains(storeName)){
				deferred.resolve(null);
			}else{
				var txn = db.transaction(storeName, READ_ONLY),
					store = txn.objectStore(storeName),
					req = store.openCursor(),
					data = [];
				_regErrorHandler(req,deferred);	
				txn.oncomplete = function() {
					deferred.resolve(data);
				};
				req.onsuccess = function(event) {
					var result = event.target.result;
					if(result && result !== null) {
						data.push(result.value);
						result.continue();
					}
				};
			}
		});
		return deferred.promise;
	}
	function getRangeData(storeName, start, end) {
		var deferred = Q.defer();
		getDB().then(function(db) {
			
		}).catch(function(err) {
			output.error(err.name+':['+err.code+']'+err.message);
			deferred.reject(err);
			return err;
		});
		
		var deferred = Q.defer();
		_getDbObj(function(db){
			if(!db.objectStoreNames(storeName)) {
				deferred.resolve(null);
			}else{
				var txn = db.transaction(storeName, READ_ONLY),
					store = txn.objectStore(storeName),
					range = IDBKeyRange.bound(start, end),
					req = store.openCursor(range),
					data = [];
				_regErrorHandler(req,deferred);	
				txn.oncomplete = function() {
					deferred.resolve(data);
				};	
				req.onsuccess = function(event) {
					var cursor = event.target.result;
					if(cursor) {
						dataArr.push(cursor.value);
						cursor.continue();
					}
				};
			}
		});
		return deferred.promise;
	}
	function getData(storeName,id){
		var deferred = Q.defer();
		_getDbObj(function(db){
			if(!db.objectStoreNames.contains(storeName)){
				deferred.resolve(null);
			}else{
				var txn = db.transaction(storeName, READ_ONLY),
					store = txn.objectStore(storeName),
					req = store.get(id),
					data = null;
				_regErrorHandler(req,deferred);	
				txn.oncomplete = function() {
					deferred.resolve(data);
				};
				req.onsuccess = function(event) {
					data = event.target.result;
				};
			}
		});
		return deferred.promise;
	}
	function getDataByIndex(storeName,indexName,indexValue){
		var deferred = Q.defer();
		_getDbObj(function(db){
			if(!db.objectStoreNames.contains(storeName)){
				deferred.resolve(null);
			}else{
				var txn = db.transaction(storeName, READ_ONLY),
					store = txn.objectStore(storeName),
					index = store.index(indexName),
					req = index.get(indexValue);
					data = null;
				_regErrorHandler(req,deferred);		
				txn.oncomplete = function() {
					deferred.resolve(data);
				};
				req.onsuccess = function(event) {
					data = event.target.result;
				};
			}
		});
		return deferred.promise;
	}

	function addOneData(storeName, data) {
		var deferred = Q.defer();
		getDB().then(function(db){
			if(db.objectStoreNames.contains(storeName)){
				var txn, store,req,res;
				txn = db.transaction(storeName, READ_WRITE);
				store = txn.objectStore(storeName);
				req = store.add(data);
				txn.oncomplete = function() {
					deferred.resolve(res);
				};
				req.onsuccess = function(event) {
					res = event.target.result;
				};
				req.onerror = function(event) {
					var err = event.target.error;
					output.error(err.name+':['+err.code+']'+err.message);
					deferred.reject(err);
				};
			}else{
				deferred.reject({code:0,message:'存储对象不存在'});
				
				
			}
		}).catch(function(err){
			output.error(err.name+':['+err.code+']'+err.message);
			deferred.reject(err);
		});
		return deferred.promise;
	}
	function putOneData(storeName, data) {
		var deferred = Q.defer();
		getDB().then(function(db){
			if(db.objectStoreNames.contains(storeName)){
				var txn, store,req,res;
				txn = db.transaction(storeName, READ_WRITE);
				store = txn.objectStore(storeName);
				req = store.put(data);
				txn.oncomplete = function() {
					deferred.resolve(res);
				};
				req.onsuccess = function(event) {
					res = event.target.result;
				};
				req.onerror = function(event) {
					var err = event.target.error;
					output.error(err.name+':['+err.code+']'+err.message);
					deferred.reject(err);
				};
			}else{
				deferred.reject({code:0,message:'存储对象不存在'});
			}
		}).catch(function(err){
			output.error(err.name+':['+err.code+']'+err.message);
			deferred.reject(err);
		});
		return deferred.promise;
	}
	
 	function saveData(storeName, data, keyName) {
		var deferred = Q.defer();
		_getDbObj(function(db){
			if(db.objectStoreNames.contains(storeName)){
				_putDataToDb(db,storeName,data,deferred);
			}else{
				output.error('objectStore "'+storeName+'" is not exist');
				deferred.reject({
					code:0,
					message:'objectStore "'+storeName+'" is not exist'
				});
			}
		});
		return deferred.promise;
	}
 	function deleteData(storeName, id) {
 		var deferred = Q.defer();
		_getDbObj(function(db){
			if(db.objectStoreNames.contains(storeName)){
				var txn = db.transaction(storeName, READ_WRITE),
					store = txn.objectStore(storeName),
					req = store.delete(id),
					data = 0;
				_regErrorHandler(req,deferred);
				txn.oncomplete = function() {
					deferred.resolve(data);
				};
				req.onsuccess = function(event) {
					data = event.target.result;
				};
			}else{
				deferred.resolve(0);
			}
		});
		return deferred.promise;
	}
 	function deleteAll(storeName) {
 		var deferred = Q.defer();
		_getDbObj(function(db){
			if(db.objectStoreNames.contains(storeName)){
				var txn = db.transaction(storeName, READ_WRITE),
					store = txn.objectStore(storeName),
					req = store.clear()
					data = 0;
				_regErrorHandler(req,deferred);	
				txn.oncomplete = function() {
					deferred.resolve(data);
				};
				req.onsuccess = function(event) {
					data = event.target.result;
				};
			}else{
				deferred.resolve(0);
			}	
		});
		return deferred.promise;
	}

	var data = Object.freeze({
		getAllData: getAllData,
		getData: getData,
		getDataByIndex: getDataByIndex,
		getRangeData: getRangeData,
		saveData: saveData,
		addOneData: addOneData,
		putOneData: putOneData,
		deleteData: deleteData,
		deleteAll: deleteAll
	});

	var _extends = Object.assign || function(target) {
		for(var i = 1; i < arguments.length; i++) {
			var source = arguments[i];
			for(var key in source) {
				if(Object.prototype.hasOwnProperty.call(source, key)) {
					target[key] = source[key];
				}
			}
		}
		return target;
	};
 	var jdb = _extends({}, db, store, data);
	
	window.jdb = jdb;
}(mui));