(function($){
	var getTranslateY = function(el){
		var transform = el.style.transform||el.style['webkitTransform']||'translate3d(0px,0px,0px)';
		var transformY = transform.match(/translate3d\(.+,\s*((\-|\+)?\d+(\.\d+)?)px,.+\)/i)[1];
		var offset = isNaN(transformY)?0:Number(transformY);
		return offset;
	};
	var getAngle = function (p1, p2, props) {
		if(!props) {
			props = ['clientX', 'clientY'];
		}
		var x = p2[props[0]] - p1[props[0]];
		var y = p2[props[1]] - p1[props[1]];
		var angle = Math.atan2(y, x) * 180 / Math.PI;
		if (angle < -70 && angle > -110) {
			return 'up';
		} else if (angle >= 70 && angle < 110) {
			return 'down';
		} else if (angle >= 160 || angle <= -160) {
			return 'left';
		} else if (angle >= -35 && angle <= 10) {
			return 'right';
		}
		return null;
	};
	$.fn.JPull = function(){
		var apis = [];
		this.each(function() {
			var id = this.getAttribute('data-pull');
			if(id){
				 apis.push($.data[id]);
			}else{
				apis.push({});
			}
		});
		return apis.length === 1 ? apis[0] : apis;
	};

	$.fn.setJPullToRefresh=function(options){
		var apis = [];
		this.each(function() {
			var api = null;
			var self = this;
			var id = self.getAttribute('data-pull');
			if (!id) {
				id = ++$.uuid;
				$.data[id] = api = new jpull(self, options);
				self.setAttribute('data-pull', id);
			} else {
				api = $.data[id];
			}
			apis.push(api);
		});
		return apis.length === 1 ? apis[0] : apis;
	};

	var jpull = $.Class.extend({
		init:function(el,options){
			this.el = el;
			this.pulling = false;
			this.stop = false;
			this.enablePullUp = false;
			this.enablePullDown = false;
			
			this.options = $.extend(true, {
		    		pulldownMoveStart:0,
		    		pulldownAngleRate:1,
		    		up:{
		    			offset:40,//距离底部高度(到达该高度即触发)
		    			auto: false,
					show: true,
					contentdown: '上拉显示更多',
					contentrefresh: '正在加载...',
					contentnomore: '没有更多数据了',
					callback: false
		    		},
		    		down:{
		    			offset: 60,
		    			startTips: '下拉刷新',
					canPullTips: '松开刷新',
					refreshTips: '正在刷新',
					endPullTips: '刷新完成',
					callback: false,
		    		}
		    },options);
		    this.enablePullUp = !!this.options.up.callback;
		    this.enablePullDown = !!this.options.down.callback;

		    this.options._pulldownMoveStart = this.options.pulldownMoveStart;
		    this.options._pulldownAngleRate = this.options.pulldownAngleRate;
		    
		    this.initUI();
		    this.initEvents();
		},
		initUI:function(){
			if(this.options.up){
				var pullUpTips = document.createElement('div');
				pullUpTips.className = 'joy-pullup-tips';
				pullUpTips.innerHTML = [
					'<div class="tip-message">',
						'<i class=" mui-spinner"></i>',
						'<span class="message-text">',this.options.up.contentdown,'</span>',
					'</div>'
				].join('');
				this.el.appendChild(pullUpTips);
				this.pullUpTips = pullUpTips;
				this.pullUpIcon = pullUpTips.querySelector('i');
				this.pullUpMessage = pullUpTips.querySelector('.message-text');
			}
			if(this.options.down){
				var pullDownTips = document.createElement('div');
				pullDownTips.className = 'joy-pull-tips';
				pullDownTips.innerHTML = [
					'<div class="tip-message">',
						'<i class="mui-icon mui-icon-arrowthindown"></i>',
						'<span class="message-text">',this.options.down.startTips,'</span>',
					'</div>'
				].join('');
				this.el.appendChild(pullDownTips);
				this.pullDownTips = pullDownTips;
				this.pullMessage = pullDownTips.querySelector('.tip-message');
				this.pullDownIcon = pullDownTips.querySelector('i');
				this.pullDownMessage = pullDownTips.querySelector('.message-text');
			}
		},
		initEvents:function(detach){
			if(!this.options.up.callback&&!this.options.down.callback) return;
			var action = detach ? 'removeEventListener' : 'addEventListener';
			this.el[action]('touchstart',this);
			this.el[action]('touchmove',this);
			this.el[action]('scrollend',this);
			this.el[action]('touchend',this);
		},
		handleEvent:function(e){
			if (this.stop) {
				this.reset();
				return;
			}
			switch (e.type) {
				case $.EVENT_START:
					this.start(e);
					break;
				case $.EVENT_MOVE:
					this.pull(e);
					break;
				case $.EVENT_CANCEL:
				case $.EVENT_END:
					this.end(e);
					break;
				case 'scrollend':
					this.fpull(e);
					break;
			}
		},
		start:function(e){
			this.pullStart = e.touches[0];
		},
		pull:function(e){
			this.moveX = e.touches[0].clientY - this.pullStart.clientX;
			this.moveY = e.touches[0].clientY - this.pullStart.clientY;
			var direction = getAngle(this.pullStart,e.touches[0],['pageX','pageY']);
			var offset = getTranslateY(this.el);
			if(this.enablePullDown && direction=='down' && offset>=0 && this.moveY>this.options.pulldownMoveStart){
			    	//Math.abs(this.moveY)>Math.abs(this.options.pulldownAngleRate*this.moveX)
				if(!this.pulling) {
					this.pullDirection='down';
				}
				this.showPullDown();
				e.stopPropagation();
				return;
			}
			if(this.enablePullUp && this.moveY<0){	
				this.runPullUp();
				return;
			}
		},
		fpull:function(e){
			if(this.enablePullUp){
				this.runPullUp();
			}
		},
		end:function(e){
			if(this.pullDirection=='down' && this.enablePullDown){
				this.runPullDown();
			}else{
				var offset = getTranslateY(this.el);
				if(offset>0){
					this.el.style['webkitTransform'] = 'translate3d(0px,0px,0px)';
					this.el.style['webkitTransitionTimingFunction'] = 'cubic-bezier(0.1, 0.57, 0.1, 1)';
					this.el.style['webkitTransitionDuration'] = '300ms';
				}
			}
		},
		showPullDown:function(){
			var offset = (this.moveY-this.options.pulldownMoveStart)/3;	
			this.el.style['webkitTransform'] = 'translate3d(0px,'+offset+'px,0px)';
			this.el.style['webkitTransitionTimingFunction'] = 'cubic-bezier(0.1, 0.57, 0.1, 1)';
			this.el.style['webkitTransitionDuration'] = '0ms';
			if(this.pulling) return;
			$.trigger(this.el,'pulldowning');
			this.pullDownTips.classList.remove('mui-hidden');
			if(offset>=this.options.down.offset){
				this.pullMessage.className='tip-message pullover';
				this.pullDownIcon.className='mui-icon mui-icon-arrowthinup';
				if(this.pullDownMessage.innerHTML!=this.options.down.canPullTips){
					this.pullDownMessage.innerHTML=this.options.down.canPullTips;
				}
			}else{
				this.pullMessage.className='tip-message pullstart';
				this.pullDownIcon.className='mui-icon mui-icon-arrowthindown';
				if(this.pullDownMessage.innerHTML!=this.options.down.startTips){
					this.pullDownMessage.innerHTML=this.options.down.startTips;
				}
			}
			
		},
		runPullDown:function(){
			if(this.pulling) return;
			var offset = getTranslateY(this.el);
			if(offset>=this.options.down.offset){
				console.log('pull offset:'+offset);
				this.pullMessage.className='tip-message refrenshing';
				this.pullDownIcon.className='mui-icon mui-icon-spinner-cycle mui-spin';
				this.pullDownMessage.innerHTML=this.options.down.refreshTips;
				
				this.el.style['webkitTransform'] = 'translate3d(0px,40px,0px)';
				this.el.style['webkitTransitionTimingFunction'] = 'cubic-bezier(0.1, 0.57, 0.1, 1)';
				this.el.style['webkitTransitionDuration'] = '300ms';
				
				if(!this.pulling){
					this.pulling = true;
					var callback = this.options.down.callback;
					if(callback) callback.call(this);
					this.pullDirection = '';
					$.trigger(this.el,'pulldownend');
				}
			}else if(offset>0){
				this.el.style['webkitTransform'] = 'translate3d(0px,0px,0px)';
				this.el.style['webkitTransitionTimingFunction'] = 'cubic-bezier(0.1, 0.57, 0.1, 1)';
				this.el.style['webkitTransitionDuration'] = '300ms';
			}
		},
		runPullUp:function(){
			if(this.pulling) return;
			var offset = getTranslateY(this.el);
			var margin = this.el.offsetHeight - this.el.parentElement.offsetHeight + offset;
			if(margin<this.options.up.offset){
				this.pulling = true;
				this.pullUpTips.classList.add('runing');
				this.pullUpMessage.innerHTML = this.options.up.contentrefresh;
				var callback = this.options.up.callback;
				if(callback) callback.call(this);
			}
		},
		setStop:function(flag){
			this.stop = !!flag;
		},
		setOptions:function(opts){
			this.options.pulldownMoveStart = this.options._pulldownMoveStart = opts.pulldownMoveStart||0;
			this.options.pulldownAngleRate = this.options._pulldownAngleRate = opts.pulldownAngleRate||1;
		},
		reset:function(){
			this.options.pulldownMoveStart = this.options._pulldownMoveStart;
			this.options.pulldownAngleRate = this.options._pulldownAngleRate;
		},
		disablePullupToRefresh:function(){
			this.enablePullUp = false;
			this.initEvents(true);
		},
		refreshPullupToRefresh:function(){
			this.enablePullUp = true;
			this.initEvents();
		},
		endPullUpToRefresh:function(nomore){
			if(nomore){
				this.enablePullUp = false;
				this.pullUpMessage = this.options.up.contentnomore;
			} 
			this.pullUpTips.classList.remove('runing');
			this.pullUpMessage = this.options.up.contentdown;
			this.pulling = false;
		},
		endPullDownToRefresh:function(tips){
			this.pulling = false;
			this.pullMessage.className='tip-message refreshed';
			this.pullDownIcon.className='mui-icon mui-icon-checkmarkempty';
			if(!isNaN(tips)){
				this.pullDownMessage.innerHTML=tips?'为您更新'+tips+'条内容':'没有更新内容';
			}else if(typeof tips === 'string'){
				this.pullDownMessage.innerHTML=tips;
			}else{
				this.pullDownMessage.innerHTML=this.options.down.endPullTips;
			}
			var This = this;
			setTimeout(function(){
				var offset = getTranslateY(This.el);
				if(offset>0){
					This.el.style['webkitTransform'] = 'translate3d(0px,0px,0px)';
					This.el.style['webkitTransitionTimingFunction'] = 'cubic-bezier(0.1, 0.57, 0.1, 1)';
					This.el.style['webkitTransitionDuration'] = '300ms';
				}
				This.pullDownTips.classList.add('mui-hidden');
				This.pullMessage.className='tip-message pullstart';
				This.pullDownIcon.className='mui-icon mui-icon-arrowthindown';
				This.pullDownMessage.innerHTML=This.options.down.startTips;
			},1200);
		}
		
	});

})(mui);

