/* 客服系统消息加密 JS */
var IM_MSG_ENCRYPT = {
    /**IM消息加密
	* @param {Object} content 原始数据
	* @param {Number} clientId 存放AES对象的key
	*/
    msg_encrypt: function (content, clientId) {
        var imaes;
		try {
			imaes = this.getKey(clientId);
			return this._getCryptMsg(content, imaes);
		} catch (e) {
			return this._getCryptMsg(content, imaes);
		}
    },
	/**
	 * @param {String} content 数据内容
	 * @param {Object} imaes aes对象
	 */
	_getCryptMsg: function(content, imaes){
		var encrypt_type, keyVer,
		timestamp = new Date().getTime(),
		nonce = this.uuid(6),signature;
		if (typeof content !== 'string') {
			content = JSON.stringify(content);
		}
		content = imaes.aes_encrypt(content);
		encrypt_type = "AES";
		signature = imaes.aes_encrypt(nonce + timestamp);
		keyVer = imaes.version;
		return {
		    encrypt_type: encrypt_type,
		    timestamp: timestamp,
		    nonce: nonce,
		    signature: signature,
		    key_ver: keyVer,
		    data: content
		};
	},
    /**IM消息解密
	* @param {String} encryptContent 原始数据
	* @param {Number} clientId 存放AES对象的key
	*/
    msg_decrypt: function (encryptContent, clientId) {
		var imaes, encryptType;
		try {
			imaes = this.getKey(clientId);
			if (typeof encryptContent === 'string') {
				encryptContent = JSON.stringify(encryptContent);
			}
			encryptType = encryptContent && encryptContent.encrypt_type;
			//兼容老版本
			if (encryptType == "AES") {
			    // AES 解密
			    var timestamp = encryptContent.timestamp;
			    var nonce = encryptContent.nonce;
			    var signature = encryptContent.signature;
			    // 验证时效
			    var lag = new Date().getTime() - timestamp;
			    if (lag < -1000 * 60 || lag > 1000 * 60) {
			        throw new Error("消息已失效");
			    }
			    // 验证签名
			    if (signature != imaes.aes_encrypt(nonce + timestamp)) {
			        throw new Error("签名错误");
			    }
			    var aesDecrypt = imaes.aes_decrypt(encryptContent.data);
			    return JSON.parse(aesDecrypt);
			} else {
			    return encryptContent;
			}
		} catch (e) {
			return encryptContent;
		}
    },

    /* —————————————————————————————————————————————————————————————————————————————————————————————————————————————— */
    key_data: {},
    /*初始化客户端*/
    init: function (clientId) {
        // ---- 从服务器重新获取key
        var base64Key = this.changeKey(clientId);
        this.key_data[clientId] = base64Key ? new IM_AES(base64Key) : base64Key;
    },
    getKey: function (clientId) {
        // ---- 从缓存里获取key
        var data = this.key_data[clientId];
        if (!data) {
            throw new Error("客户端未加密初始化");
        }
        return data;
    },
    cleanKey: function (clientId) {
        delete this.key_data[clientId];
    },
    /*向服务器兑换AES密钥*/
    changeKey: function (clientId) {
        // RSA非对称加密
        // 支持 512、1024、2048、4096
        // key 越长, 生成密钥和加密时间越长
        var crypt = new JSEncrypt({default_key_size: 512});

        // 获取公钥
        var public_key = crypt.getPublicKey();

        // 获取私钥
        var private_key = crypt.getPrivateKey();
        // 兑换AES密钥
        var aesKey;
        $.ajax({
            url: app._customURL + "/im/encrypt/aes/keychange",
            async: false,
            type: "POST",
            data: {
                pukey: public_key,
                clientId: clientId
            },
			success: function(res){
				if (res && res.code === '000000') {
					aesKey = res.data.aesKey;
				} else {
					mui.confirm('获取加密密钥失败，不能进行客服对话', '', ['退出', '了解'], function(res){
						if (res.index == 0) {
							mui.back();
						}
					}, 'div')
				}
			}
        });
        return aesKey ? crypt.decrypt(aesKey) : null;
    },
    uuid: function (len, radix) {
        var chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.split('');
        var uuid = [], i;
        radix = radix || chars.length;

        if (len) {
            // Compact form
            for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random() * radix];
        } else {
            // rfc4122, version 4 form
            var r;

            // rfc4122 requires these characters
            uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
            uuid[14] = '4';

            // Fill in random data.  At i==19 set the high bits of clock sequence as
            // per rfc4122, sec. 4.1.5
            for (i = 0; i < 36; i++) {
                if (!uuid[i]) {
                    r = 0 | Math.random() * 16;
                    uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
                }
            }
        }
        return uuid.join('');
    }
};
/*加密操作对象*/
var IM_AES = (function () {
    var properties = {
        mode: CryptoJS.mode.ECB,
        padding: CryptoJS.pad.Pkcs7
    };


    function IM_AES(base64Key) {
        console.log("AES BASE64 KEY : " + base64Key);
        this.aesKey = CryptoJS.enc.Base64.parse(base64Key);
    }

    IM_AES.prototype.aes_encrypt = function (context) {
        context = CryptoJS.enc.Utf8.parse(context);
        var encryptMsg = CryptoJS.AES.encrypt(context, this.aesKey, properties);
        return encryptMsg.toString();

    };

    IM_AES.prototype.aes_decrypt = function (encryptMsg) {
        var decrypted = CryptoJS.AES.decrypt(encryptMsg, this.aesKey, properties);
        return CryptoJS.enc.Utf8.stringify(decrypted);
    };
    return IM_AES;
}());

