/**
 * Created by ezgoing on 14/9/2014.
 */
'use strict';
var cropbox = function(options){
	var EVENT_PINCH_START = 'pinchstart';
	var EVENT_PINCH = 'pinch';
	var EVENT_PINCH_END = 'pinchend';
	if ('ongesturestart' in window) {
		EVENT_PINCH_START = 'gesturestart';
		EVENT_PINCH = 'gesturechange';
		EVENT_PINCH_END = 'gestureend';
	}
	var _extends = Object.assign || function(target) {
		for(var i = 1; i < arguments.length; i++) {
			var source = arguments[i];
			for(var key in source) {
				if(Object.prototype.hasOwnProperty.call(source, key)) {
					target[key] = source[key];
				}
			}
		}
		return target;
	};
	var defaults = {
		maxZoom: 4,
		minZoom: .1,
		outType:'image/png'
	};
	options = _extends({},defaults,options);
	var pixelRatio = devicePixelRatio || 1;
    var el = document.querySelector(options.imageBox),
    obj = {
        state : {},
        scale : 1,
        currentScale : 1,
        options : options,
        imageBox : el,
        thumbBox : el.querySelector(options.thumbBox),
        spinner : el.querySelector(options.spinner),
        image : new Image(),
        getDataURL: function () {
            var width = this.thumbBox.clientWidth,
                height = this.thumbBox.clientHeight,
                canvas = document.createElement("canvas"),
                context = canvas.getContext("2d"),
                dim = el.style.backgroundPosition.split(' '),
                size = el.style.backgroundSize.split(' '),
                dcx = el.clientWidth/2 - width/2,
                dcy = el.clientHeight/2 - height/2,
                dx = parseInt(dim[0]) - dcx,
                dy = parseInt(dim[1]) - dcy,
                dw = parseInt(size[0]),
                dh = parseInt(size[1]),
                ow = parseInt(this.image.width),
                oh = parseInt(this.image.height),
                sc = ow/dw,
                rx = dx*sc,
                ry = dy*sc;

            var imageData;
           	if(options.scale==false){
                var cl = Math.max(rx,0),
                    ct = Math.max(ry,0),
                    cr = Math.min(rx+ow,width*sc),
                    cb = Math.min(ry+oh,height*sc),
                    sx = Math.max(0-rx,0),
                    sy = Math.max(0-ry,0),
                    cw = cr-cl,
                    ch = cb-ct;
                canvas.width = cw;
                canvas.height = ch;
           		context.drawImage(this.image, sx, sy, ow, oh, 0, 0, ow, oh);
           	}else{
           	    sc = options.cropWidth/width;
           	    rx = dx*sc;
                ry = dy*sc;
                sx = rx>0 ? 0 : 0-rx;
                sy = ry>0 ? 0 : 0-ry;
                var fx = rx>0 ? rx : 0;
                var fy = ry>0 ? ry : 0;
                canvas.width = options.cropWidth;
                canvas.height = options.cropHeight;

                context.drawImage(this.image, 0, 0, ow, oh, rx, ry, dw*sc, dh*sc);
                //context.drawImage(this.image, sx, sy, ow, oh, fy, fy, dw*sc, dh*sc);
           	}
            imageData = canvas.toDataURL(options.outType,1);
            return imageData;
        },
        getBlob: function(){
            var imageData = this.getDataURL();
            var b64 = imageData.replace(/data:(.*);base64,/,'');
            var binary = atob(b64);
            var array = [];
            for (var i = 0; i < binary.length; i++) {
                array.push(binary.charCodeAt(i));
            }
            return  new Blob([new Uint8Array(array)], {type: options.outType});
        },
        zoomIn: function (per){
        	var theScale = per ? (1+per):1.1;
            setZoom(theScale);
        },
        zoomOut: function (per){
            var theScale=per ? (1-per):0.9;
            setZoom(theScale);
        }
    },
    attachEvent = function(node, event, cb){
        if (node.attachEvent)
            node.attachEvent('on'+event, cb);
        else if (node.addEventListener)
            node.addEventListener(event, cb);
    },
    detachEvent = function(node, event, cb){
        if(node.detachEvent) {
            node.detachEvent('on'+event, cb);
        }else if(node.removeEventListener) {
            node.removeEventListener(event, render);
        }
    },
    stopEvent = function (e) {
    	if(window.event) 
        	e.cancelBubble = true;
        else 
        	e.stopImmediatePropagation();
    },
    setZoom = function(thisScale){
		obj.scale = thisScale * obj.currentScale;
		if (obj.scale > obj.options.maxZoom) {
			obj.scale = obj.options.maxZoom;
			//obj.scale = obj.options.maxZoom - 1 + Math.pow((obj.scale - obj.options.maxZoom + 1), 0.5);
		}
		if (obj.scale < obj.options.minZoom) {
			obj.scale = obj.options.minZoom;
			//obj.scale = obj.options.minZoom + 1 - Math.pow((obj.options.minZoom - obj.scale + 1), 0.5);
		}
		if(obj.scale == obj.currentScale) return;
		var dim = el.style.backgroundPosition.split(' ');
		var cx = parseFloat(dim[0]),cy = parseFloat(dim[1]);
		var ox = el.clientWidth/2,oy=el.clientHeight/2;
		var dx = ox - (ox-cx)*thisScale;
		var dy = oy - (oy-cy)*thisScale;
        var w =  parseInt(obj.image.width/pixelRatio)*(obj.scale||1);
        var h =  parseInt(obj.image.height/pixelRatio)*(obj.scale||1);
        el.setAttribute('style',
            'background-image: url(' + obj.image.src + '); ' +
            'background-size: ' + w +'px ' + h + 'px; ' +
            'background-position: ' + dx +'px ' + dy + 'px; ' +
            'background-repeat: no-repeat');
        obj.currentScale = obj.scale;  
    },
    zoomImage = function(e){
        var evt = window.event || e;
		var delta = evt.detail ? evt.detail * (-120) : evt.wheelDelta;
		var scale = delta > -120 ? 1.1 : 0.9;
		setZoom(scale);
    },
    setBackground = function(){
    	var w =  parseInt(obj.image.width/pixelRatio)*(obj.scale||1);
        var h =  parseInt(obj.image.height/pixelRatio)*(obj.scale||1);

        var pw = (screen.width - w) / 2;
        var ph = (el.clientHeight - h) / 2;
		var pos = el.style.backgroundPosition|| pw + 'px ' + ph + 'px';
		
        el.setAttribute('style',
            'background-image: url(' + obj.image.src + '); ' +
            'background-size: ' + w +'px ' + h + 'px; ' +
            'background-position: ' + pos + ' ;' +
            'background-repeat: no-repeat');   
    },
    
    imgMoveStart = function(e){
    	console.log('imgMoveStart');
    	obj.state.dragable = true;
        if(e.type == 'touchstart') e = e.touches[0];
        obj.state.mouseX = e.clientX;
        obj.state.mouseY = e.clientY;
    },
    imgMove = function(e) {
    	if(e.touches && e.touches.length>1) return;
        if (!obj.state.dragable) return;
    	if(e.type == 'touchmove') e = e.touches[0];
        var x = e.clientX - obj.state.mouseX;
        var y = e.clientY - obj.state.mouseY;

        var bg = el.style.backgroundPosition.split(' ');
        var bgX = x + parseFloat(bg[0]);
        var bgY = y + parseFloat(bg[1]);
        el.style.backgroundPosition = bgX +'px ' + bgY + 'px';

        obj.state.mouseX = e.clientX;
        obj.state.mouseY = e.clientY;
        console.log('imgMove');
    },
    imgMoveEnd = function(e){
    	console.log('imgMoveEnd');
    	obj.state.dragable = false;
        if (e.touches && e.touches.length<2) obj.state.isGesturing = false;
    },
    
    onPinchstart = function(e){
    	console.log('onPinchstart');
    	obj.state.isGesturing = true;
    	obj.state.mouseX = e.detail ? e.detail.center.x : el.clientWidth/2;
        obj.state.mouseY = e.detail ? e.detail.center.y : el.clientHeight/2;
        var dim = el.style.backgroundPosition.split(' ');
		obj.state.pos = {left: parseFloat(dim[0]),top:parseFloat(dim[1])};
		console.log(obj.state.pos);
    },
    onPinch = function(e){
    	if (!obj.state.isGesturing) return;
    	console.log('onPinch Start:',obj.scale);
    	var thisScale = e.detail ? e.detail.scale : e.scale;
		obj.scale = thisScale * obj.currentScale;
		if (obj.scale > obj.options.maxZoom) {
			obj.scale = obj.options.maxZoom;
			//obj.scale = obj.options.maxZoom - 1 + Math.pow((obj.scale - obj.options.maxZoom + 1), 0.5);
			return;
		}
		if (obj.scale < obj.options.minZoom) {
			obj.scale = obj.options.minZoom;
			//obj.scale = obj.options.minZoom + 1 - Math.pow((obj.options.minZoom - obj.scale + 1), 0.5);
			return;
		}
		var dx = obj.state.mouseX - (obj.state.mouseX-obj.state.pos.left)*thisScale;
		var dy = obj.state.mouseY - (obj.state.mouseY-obj.state.pos.top)*thisScale;
        el.style.backgroundPosition = dx +'px ' + dy + 'px';
        setBackground();
        console.log('onPinch End:',obj.scale);
    },
    onPinchend = function(e){
    	obj.scale = Math.max(Math.min(obj.scale, obj.options.maxZoom), obj.options.minZoom);
		obj.currentScale = obj.scale;
		obj.state.isGesturing = false;
		console.log('onPinchend');
    };
	
    obj.spinner.style.display = 'block';
    obj.image.onload = function() {
        obj.spinner.style.display = 'none';
        var imgWidth = parseInt(obj.image.width/pixelRatio);
        var scaleWidth = screen.width;
        var theScale = scaleWidth/imgWidth;
        var scaleHeight = theScale * parseInt(obj.image.height/pixelRatio);
        obj.scale = obj.currentScale = theScale;
        el.style.backgroundPosition = '';
        el.style.backgroundSize = scaleWidth+'px '+scaleHeight+'px';
        
        setBackground();
        attachEvent(el,EVENT_PINCH_START, onPinchstart);
		attachEvent(el,EVENT_PINCH, onPinch);
		attachEvent(el,EVENT_PINCH_END, onPinchend);
        
		attachEvent(el, 'touchstart', function(e){
			e.preventDefault();
			imgMoveStart(e);
		});
        attachEvent(el, 'touchmove', function(e){
        	e.preventDefault();
			imgMove(e);
        });
        attachEvent(document.body, 'touchcancel',function(e){
        	imgMoveEnd(e);
        });
        attachEvent(document.body, 'touchend',function(e){
        	imgMoveEnd(e);
        });
        attachEvent(el,'drag', function(e) {
			if (obj.state.isGesturing) {
				e.stopPropagation();
			}
		});
		attachEvent(el,'doubletap', function(e) {
			stopEvent(e);
			var position = e.detail.center;
            var bg = el.style.backgroundPosition.split(' ');
			var bgX = position.x - parseInt(bg[0])*obj.scale;
            var bgY =  position.y - parseInt(bg[1])*obj.scale;
            
			obj.scale = obj.currentScale = 1;
            el.style.backgroundPosition = bgX +'px ' + bgY + 'px';
			setBackground();
		});
        attachEvent(el, 'mousedown', function(e){
        	stopEvent(e);
			imgMoveStart(e);
        });
        attachEvent(el, 'mousemove', function(e){
        	stopEvent(e);
			imgMove(e);
        });
        attachEvent(document.body, 'mouseup', function(e){
        	stopEvent(e);
        	imgMoveEnd(e);
        });
    	var mousewheel = (/Firefox/i.test(navigator.userAgent))? 'DOMMouseScroll' : 'mousewheel';
        attachEvent(el, mousewheel, zoomImage);
    };
    obj.image.src = options.imgSrc;
    attachEvent(el, 'DOMNodeRemoved', function(){
    	detachEvent(document.body, 'DOMNodeRemoved', imgMouseUp)
	});

    return obj;
};
