(function (factory) {
    if (typeof exports === 'object') {
        // Node/CommonJS
        module.exports = factory();
    } else if (typeof define === 'function' && define.amd) {
        // AMD 
        define(factory);
    } else {
        // Browser globals (with support for web workers)
        var glob;
        try {
            glob = window;
        } catch (e) {
            glob = self;
        }
        glob.content = factory();
    }
}(function(){
 	var bar = {};
 	var colors = ['#f63','#f90','#3c6', '#069', '#27727B','#FE8463'];
 	
	function pie(dom, data){
		var cont = null;
		dom.style.position = 'relative';
		dom.style.paddingBottom = '20px';
		var cloneDom = null;
		var tempDom = null;
		try {
			var parseData = JSON.parse(data.content);				
		} catch(e){
			console.log(e)
		}
		cont = document.createElement('div');
		cont.style.height = (document.documentElement.clientWidth - 120) + 'px';
		cont.classList.add('canvas');
		cont.data = data.content;
	    dom.appendChild(cont);
    	drawCont(dom, cont, parseData);	
    	cont = null;
	}
	function drawCont(targetDom, cont, val, isFullScreen){
		if (typeof val == 'string') {
			try {
				val = JSON.parse(val);
			} catch (e) {
				return;
			}
		}
		
		setTimeout(function(){
			moduleHandler.addXY({x: val.dataList[0].title.subtext, y: val.dataList[0].verticalLabel}, targetDom);
			moduleHandler.addLegend(val, targetDom);
			if (!isFullScreen) {
				moduleHandler.addRemarks(val.dataList[0].remarks, targetDom);
				// 竖向标题
				moduleHandler.addBetween('双击图表试试', 'vertical-label');
			} else {
				cont.style.height = (cont.offsetHeight - moduleHandler.iHeight - 30) +'px';
			}
			drawCharts(cont, val, isFullScreen);
		}, 0);
		
		var moduleHandler = {
			iHeight: 0,
			/**
			 * 插入两侧标题
			 * @param {Object} text
			 * @param {Object} className
			 */
			addBetween: function (text, className){			
				if (!text) return;
				var oDom = document.createElement('div');
				oDom.innerHTML = text;
				
				if (isFullScreen) {
					oDom.style.fontSize = '16px'
				}
				oDom.classList.add(className);	
				targetDom.appendChild(oDom);
			},
			
			/**
			 * 插入图例
			 * @param {Object} data 图表数据
			 * @param {Object} targetDom
			 */
			addLegend: function (data, targetDom){
				var wrapDom, legend;
				var colors = data.dataList[0].colors || colors;
				
				if (data.type == '1' || data.type == '2-2') return;
				
				legend = data.dataList[0].legend;
				
				// 图例标记
				if (legend && legend.length > 0) {
					wrapDom = document.createElement('div');
					wrapDom.classList.add('box-tips');	
					
					if (isFullScreen) {
						wrapDom.classList.add('nowrap');
					}
					
					mui.each(legend, function(i, val){
						tempData = document.createElement('div');
						tempData.innerHTML = '<span style="background:' + colors[i] + '"></span><em>' + val + '</em>';
						wrapDom.appendChild(tempData);
					})
					targetDom.appendChild(wrapDom);	
					this.iHeight += wrapDom.offsetHeight;
					tempData = null;
					wrapDom = null;
				}
			},
			
			/**
			 * 插入横纵坐标的label
			 * @param {Object} data
			 * @param {Object} dom
			 */
			addXY: function (data, dom){
				var wrapDom = document.createElement('div'), tempDom;
				wrapDom.classList.add('xylabel');
				for (var i in data) {
					
					if (data[i]) {
						tempDom = document.createElement('div');	
						tempDom.innerHTML = (i == 'x' ? '横轴：' : '纵轴：') + data[i];
						wrapDom.appendChild(tempDom);
					}
				}
				dom.appendChild(wrapDom);
				this.iHeight += wrapDom.offsetHeight;
				
			},
		
			/**
			 * 插入备注
			 * @param {Object} data
			 * @param {Object} dom
			 */
			addRemarks: function (data, dom){
				
				if (data && !data.content) return;
				var tempDom = document.createElement('div');
				tempDom.classList.add('remark');
				tempDom.innerHTML = '<div><label>' + data.label + '</label></div><span>' + data.content + '</span>'
				dom.appendChild(tempDom);
				tempDom = null;
			}
		};
	}
	/**
	 * @param {Array} arr 计算纵轴文字的位置和大小
	 */
	function calculateLableTextPos(arr){
			var result = getAllNumber(arr);
			var min = Math.min.apply(null, result);
			var max = Math.max.apply(null, result);
			var critical;
			var bBtn;
			if (max - min > max*.1) {
				critical = max - max*.1;
			}
			return arr.map(function(val){
				bBtn = false;
				if (Array.isArray(val)) {
					min = Math.min.apply(null, val);
					max = Math.max.apply(null, val);
					// 有大于临界值的
					if (max < critical || min < max/2) {
						bBtn = true;
					}
				}
				return {
					position: bBtn ? 'top' : 'insideTop',
					fontSize: result > 8 ? 10 : 12,
					distance: bBtn ? 2 : 30,
					color: bBtn ? '#f90' : '#fff'
				}	
			})
			
	}
	function getAllNumber(arr){
			var resultArr = [];
			if (Array.isArray(arr)) {
				arr.forEach(function(val, index){
					if (Array.isArray(val)){
						resultArr = resultArr.concat(getAllNumber(val));
					} else {
						resultArr.push(val);
					}
				})
			}
			return resultArr;
	}
	/**
	 * 序列号X轴label
	 * @param {Object} value 
	 * @param {Number} splitNum 几个字符进行分割
	 * @param {Number} num 多少组柱状图
	 * @param {Boolean} isFullScreen 是否全屏
	 */
	function formatter(value, splitNum, num, isFullScreen){
		var str;
		var count = 0;
		var result = '';
		var clientW = 0;
		var mark = 0;
		var stopNum = 3;
		
		if (!splitNum && num) {
			clientW = document.documentElement.clientWidth-80;
			splitNum = parseInt(clientW / num / 18);
		}
		if (isFullScreen) {
			splitNum = 5;
			stopNum = 4;
		}
		while (str = value.substring(count, splitNum+count)) {
			
			mark ++;
			if (mark == stopNum) {
				result = result.replace(/(?!\n){1}$/m,'...');
				break;
			};
			count += splitNum;
			result += str + (mark < 2 ? '\n' : '');
		}
		
		return result;
	}
    	
	/**
	 * 处理label
	 * @param {Object} arr
	 * @param {Object} rich
	 * @param {String} defColor 默认的颜色
	 */
	function labelHandler(arr, rich, defColor){
		var colorRE = /#([0-9a-zA-Z]{3,6})/g;
		var color = null;
		if (!Array.isArray(arr)) return;
		return arr.map(function(val, i){
			val = val.replace(/\\n/g, '\n');
			return val = val.replace(colorRE, function($1, $2){
				rich[$2] = {
					color: Array.isArray(defColor) ? defColor[i] : defColor ? defColor : '#' + $2
		    }; 
				return $2;
			});
		});
	}
		
	function drawCharts(cont, data, isFullScreen){
		var myChart = echarts.init(cont);
			var chartData = null;
			var titleData = {
		        text: data.dataList[0].title.text,
		        textStyle: {
		            color:'#595757',
		            fontSize: isFullScreen ? 20 : 16,
		            fontWeight:'200',
		        },
		        top: isFullScreen ? 10 : 0,
		        left: isFullScreen ? 'center' : 30
			},
			grid = {
		        left: 0,
		        right: isFullScreen ? 0 : '5%',
		        bottom: isFullScreen ? '5%' : '0%',
		        containLabel: true
		   },
		   yAxis = {
		        axisTick: {
	            	show: false
	            },
	            axisLine: {
			    	lineStyle: {
			    		width: 3,
			    		color: '#255c71'
			    	}
			    },
			    splitLine: {
			    	show: false
			    },
	            axisLabel: {
	            	show: true,
			    	position: 'top',
			    	offset: [0, 0],
	            	fontSize: 14,
	            	textAlign: 'center',
	            	color: '#fff',
	            	formatter: function(value, index){
						return '边界'
					}
	            },
			};
		   
		    if (data.dataList[0].colors) {
		    	colors = data.dataList[0].colors;
		    }
		    
			function turnArray(obj){
				var arr = Object.keys(obj);
				var result = [];
				
				for (var i = 0; i < arr.length; i ++) {
					result.push(obj[arr[i]]);
				}
				return result;
			}
	
			// 折线型图
			if (data.type == '1') {
				data = data.dataList[0];
				data.data = turnArray(data.data);
				var lineData = data.data[0];
				var barData = [];
				var index = -1;
				var rich = {};
				data.tipLabel = labelHandler(data.tipLabel, rich);
			
				option = {
				    backgroundColor: '#fff',
				    title : titleData,
				    grid: grid,
				    legend: {
				        data: [],
				        textStyle: {
				            color: '#ccc'
				        }
				    },
				    xAxis: {
				        data: data.xAxis,
				        axisTick: {
									show: false
								},
								axisLabel: {
					    	position: 'top',
					    	offset: [0, -40],
			            	fontSize: 14,
			            	textAlign: 'center',
			            	color: '#333',
			            	formatter: function(value, index){
								return formatter(value, isFullScreen ? 4 : 0, data.xAxis.length, isFullScreen);
							}
			            },
			            axisLine: {
					    	lineStyle: {
					    		width: 3,
					    		color: '#255c71'
					    	}
					    },
				    },
				    yAxis: yAxis,
				    series: [ {
				        name: '00',
				        type: 'bar',
				        barGap: '-100%',
				        barWidth: 1 ,
				        z: -12,
				        data: lineData
				    }, {
				        name: 'line',
				        type: 'pictorialBar',
				        symbol: 'rect',
				        itemStyle: {
				            normal: {
				                color: '#fff'
				            },
				             label: {
			                    show: true,
			                    position: 'top',
			                    color: '#f90'
			                }
				        },
				        symbolRepeat: true,
				        symbolSize: [12, 4],
				        symbolMargin: 1,
				        z: -10,
				        data: lineData
				    },
				    {
			            name: data.name,
			            type:'line',
			            smooth: true,
				        showAllSymbol: true,
				        symbol: 'emptyCircle',
				        symbolSize: 10,
				            data:lineData,
				            itemStyle: {
				                normal: {
				                    color: '#f90',
				                    label: {
				                        show: true,
				                        position: 'top',
				                        formatter: function(value){
					                    	index ++;
				                        	return data.tipLabel && data.tipLabel[index]
				                        },
				                        color: '#000',
				                        rich: rich
				                    }
				                }
				            }
					    }
				    ]
				};
			} else if (data.type == '2-1') {    //多层数据组合柱状图
				var rich = {};
				data = data.dataList[0];
				data.data = turnArray(data.data);
				data.tipLabel = labelHandler(data.tipLabel, rich);
					
				for (var i = 0; i < data.data.length; i ++) {
					data.data[i] =  {
			            name: data.data[i].name,
			            type:'bar',
			            stack: '生育',
			            barWidth: '50%',
			            data: data.data[i]
			        }
				}
			
				data.data[data.data.length-1].label = {
	                normal: {
	                    show: true,
	                    position: 'top',
	                    offset: [0,-10],
	                    formatter: function(obj){
	                    	return data.tipLabel && data.tipLabel[obj.dataIndex]
	                    },
	                    rich: rich
	                }
	            };	
	         
				option = {
					title: titleData,
					color: colors,
				    grid: grid,
				    xAxis : [
				        {
				            type : 'category',
				            data : data.xAxis,
		 					axisLabel: {
						    	position: 'top',
				            	fontSize: 14,
				            	textAlign: 'center',
				            	color: '#333',
				            	formatter: function(value, index){
									return formatter(value, isFullScreen ? 4 : 0, data.xAxis.length, isFullScreen);
								}
				            },
				            axisTick: {
				            	show: false
				            },
				            axisLine: {
						    	lineStyle: {
						    		width: 3,
						    		color: '#255c71'
						    	}
						    }
				        }
				    ],
				    yAxis : yAxis,
				    series: data.data
				};
			} else if (data.type == '2-2') { // 单组柱状图
				var index = -1;
				var rich = {};
				data = data.dataList[0];
				data.data = turnArray(data.data);
				data.tipLabel = labelHandler(data.tipLabel, rich);
				option = {
				   title: titleData,
				    toolbox: {
				        show : false,
				    },
				    tooltip : {
				    	show: false,
				        trigger: 'axis'
				    },
				    legend: {
				    	x: 'center',
				    	y: 'bottom',
				        data: data.legend
				    },
				    grid: grid,
				    color: colors,
				    calculable : true,
				    xAxis : [
				        {
								type : 'category',
								data : data.xAxis,
								axisLabel: {
									position: 'top',
									offset: [0, 40],
									fontSize: 14,
									textAlign: 'center',
									color: '#333',
									formatter: function(value, index){
										return formatter(value, isFullScreen ? 4 : 0, data.xAxis.length, isFullScreen)
									}
								},
								axisTick: {
									show: false
								},
								axisLine: {
					    	lineStyle: {
					    		width: 3,
					    		color: '#255c71'
					    	}
					    }
				      }
				    ],
				    yAxis : yAxis,
				    series : [
				        {
									name: data.name,
									type:'bar',
									data: data.data[0],
									barWidth: '50%',
									itemStyle: {
											normal: {
												 label: {
															show: true,
															position: 'top',
															fontSize: isFullScreen ? 20 : 14,
															formatter: function(value){
																index ++;
																return data.tipLabel && data.tipLabel[index];
															},
															rich: rich
													}
											}
									}
				        }
				    ]
				};  
			} else if (data.type == '2-3') { // 多组柱状图
				var rich = {};
				data = data.dataList[0];
				data.data = turnArray(data.data);
				
				var series = [];
				var index = -1;
				var noraml = calculateLableTextPos(data.data);
				var textColors = [];
				data.data.forEach(function(val, i){
					for (var j = 0; j < val.length; j ++) {
						textColors.push(noraml[i].color);
					}
				})
				data.tipLabel = labelHandler(data.tipLabel, rich, isFullScreen ? '' : textColors);
				mui.each(data.data, function(i, val){
					tempData = {
						name: '',
						type:'bar',
						data: val,
						barWidth: '20%',
						label: {
							normal: {
								show: true,
								align: 'left',
								verticalAlign: 'middle',
								rotate: isFullScreen ? 30 : 90,
								position: isFullScreen ? [10, -10] : noraml[i].position,
								distance: isFullScreen ? 0 : noraml[i].distance,
								formatter: function (value){
									index ++ ;
									return data.tipLabel && data.tipLabel[index]
								},
								rich: rich
							}
						}
			    };
					series.push(tempData);
				}) 
				
				option = {
				    title: titleData,
				    tooltip : {
				    	show: false,
				        trigger: 'axis'
				    },
				    legend: {
				    	x: 'center',
				    	y: 'bottom',
				    	data: data.legend
				    },
				    color: colors,
				    grid: grid,
				    calculable : true,
				    xAxis : [
				        {
				            type : 'category',
				            data : data.xAxis,
				            axisTick: {
				            	show: false
				            },
				            axisLine: {
						    	lineStyle: {
						    		width: 3,
						    		color: '#255c71'
						    	}
						    },
						    axisLabel: {
						    	position: 'top',
						    	offset: [0, 40],
									fontSize: 14,
									textAlign: 'center',
									color: '#333',
									formatter: function(value, index){
										return formatter(value, isFullScreen ? 4 : 0, data.xAxis.length, isFullScreen);
									}
								},
				        }
				    ],
				    yAxis : yAxis,
				    series : series
				};   
			} else {
				data = data.dataList[0];
				data.data = turnArray(data.data);
				option = {
				 
				    tooltip : {
				    	show: false,
				        trigger: 'item',
				        formatter: "{a} <br/>{b} : {c} ({d}%)"
				    },
				    legend: {
				        x : 'center',
        				y : 'bottom',
				        data: data.label
				    },
				    toolbox: {
				        show: false
				      
				    },
				    color: colors,
				    series : [
				        {
				            name: '访问来源',
				            type: 'pie',
				            radius : [40, 110],
				            center: ['50%', '40%'],
				            data: data.data,
				            roseType : 'area',
				            itemStyle: {
				                emphasis: {
				                    shadowBlur: 10,
				                    shadowOffsetX: 0,
				                    shadowColor: 'rgba(0, 0, 0, 0.5)'
				                }
				            }
				        }
				    ]
				};
			}
			myChart.setOption(option);
	}
	
	function fullScreenHandler(scope){
		var data = scope.data;
		var mask;
		var fixedDom = document.createElement('div');
		fixedDom.id='fixedDom_fullScreen';
		fixedDom.classList.add('full-screen');
		fixedDom.innerHTML = ['<div class="full-box" style="background:#fff;position:relative">',
				'<div class="con" style="width:100%;height:100%"></div>',
			'</div>'].join('');
		var cont = fixedDom.querySelector('.con');
		var fullBox = fixedDom.querySelector('.full-box');
		if(imbar.isIphoneX){
			fullBox.style.width = (document.documentElement.clientHeight - imbar.statusbarHeight - imbar.bottomSafeArea) + 'px';
			fullBox.style.height = document.documentElement.clientWidth + 'px';
			fullBox.style.marginLeft = imbar.statusbarHeight + 'px';
			fullBox.style.marginRight = imbar.bottomSafeArea + 'px';
		}else{
			fullBox.style.width = document.documentElement.clientHeight + 'px';
			fullBox.style.height = document.documentElement.clientWidth + 'px';
		}
		var oHeight = document.documentElement.clientHeight;
		plus.navigator.setFullscreen(true);
		if(plus.os.name=='iOS'){
			plus.screen.lockOrientation('landscape-secondary');
		}else{
			plus.screen.lockOrientation('landscape-primary');
		}
		var _oldBack = mui.back;
		mui.back = function(){
			closeFullScreen();
		}
		setTimeout(function(){
			mask = mui.createMask();
			mask.show();
			
			fixedDom.style.width = '100vw';
			fixedDom.style.height = '100vh';
			
			drawCont(fullBox, cont, data, true);
			var oBack = document.createElement('div');
			oBack.className = 'mui-icon mui-icon-left-nav';
			fullBox.appendChild(oBack);
			oBack.addEventListener('tap', function(e){
				e.stopPropagation();
				closeFullScreen();
			},false);
			fixedDom.addEventListener('tap', function(e){
				e.stopPropagation();
				closeFullScreen();
			},false);
			document.body.appendChild(fixedDom);
		}, 50);

		/**
		 * 关闭全屏
		 */
		function closeFullScreen(){
			plus.screen.lockOrientation("portrait-primary");
			mui.back = _oldBack;
			setTimeout(function(){
				mask.close();
				g('fixedDom_fullScreen').remove();
				plus.navigator.setFullscreen(false);
			},50);
		}
	}

	function loadStyles(){
		var styles = ['chart/css.css'];
		createStyles(styles);
	}
 	
	bar.name='chart';
	bar.show = function(el,data,config){
		bar.box = el;
		pie(el,data);
	};
	//loadStyles();
	mui('body').on('doubletap', '.canvas', function(e){
		fullScreenHandler(this);
	});
	return bar;
}));
	