(function (factory) {
    if (typeof exports === 'object') {
        // Node/CommonJS
        module.exports = factory();
    } else if (typeof define === 'function' && define.amd) {
        // AMD 
        define(factory);
    } else {
        // Browser globals (with support for web workers)
        var glob;
        try {
            glob = window;
        } catch (e) {
            glob = self;
        }
        glob.medical = factory();
    }
}(function(){
 	var bar = {};
	function setData(sbkid,data){
		initPie(data);
		bar.bxNumEl.innerText = data.medExpense;
		bar.amtNumEl.innerText = data.payTotal;
		
	}
 	function getData(sbkid){
 		var storeName = 'medical';
 		jdb.getData(storeName,sbkid).then(function(data){
 			if (!data) return;
			setData(sbkid,data);
		}).catch(function(err){
			console.error(err.message,err);
		});
		setTimeout(function(){
			joySvc.getMedicalExpense(sbkid).then(function(data){
				setData(sbkid,data);
				data.id = sbkid;
				jdb.saveData(storeName,data,'id');
			}).catch(function(err){
				console.error(err.message,err);
			});
		},50);
 	}
	function initPie(data){
		var medExpense = Number(data.medExpense);
		var payTotal = Number(data.payTotal);
		var text = (payTotal?(medExpense*100/payTotal).toFixed(2):'0');//+'%'
		bar.perEl.innerHTML = text;
		var labelNone = {
            normal : {
               label : {show : false},
               labelLine : {show : false}
            }
        };
		var chartData = [
			{name:'报销',value:medExpense,itemStyle:{emphasis:{color:'#39bfb3'}}},
			{name:'自费',value:payTotal?payTotal-medExpense:1,itemStyle:{emphasis:{color:'#eeeeee'}}}
		];
	 	var options = {
		    series : [{
		        name: '报销比例',
		        type: 'pie',
		        hoverAnimation:false,
		        startAngle:0,
		        radius : ['70%','100%'],
		        center: ['50%', '50%'],
		        color:['#39bfb3','#eeeeee'],
		        tooltip:{show:false},
		        itemStyle: labelNone,
		        data:chartData
	       }]
		};
		var chart = echarts.init(bar.chartEl);
		chart.setOption(options);		
	}
	function initUI(){
		var templateUrl = 'medical-2/tpl.html';
		mui.ajax(templateUrl,{
			dataType:'html',
			async:true,           
			success:function(html){
				bar.box.innerHTML = html;
				bar.bxNumEl = bar.box.querySelector('.data .bx div');
				bar.amtNumEl = bar.box.querySelector('.data .amt div');
				bar.chartEl = bar.box.querySelector('.pie-chart .chart');
				bar.perEl = bar.box.querySelector('.pie-chart .per b');
				
				setState();
				mui(bar.box).on('tap','.bind-bar',function(){
					if(app.checkLogin()){
						app.showSheBaoAdd();
					}else{
						app.toLogin();
					}
				});
				mui(bar.box).on('tap','.detail',function(){
					var secInfo = config.secInfo;
					if(secInfo && secInfo.barPayType!==0){
						app.showSheBaoInfo(secInfo.id,{"showTab":3});
					}else{
						mui.alert('您参保的城市暂未开通');
					}
				});
				mui(bar.box).on('tap','.analyze',function(){
					mui.openWindow({url:'_www/view/insurances.html'});
				});
			}
		});
	}
	function setState(){
		var user = config.user;
		if(user && user.securityAcctId){
			bar.box.querySelector('.data-box').classList.remove('mui-hidden');
			bar.box.querySelector('.bind-bar').classList.add('mui-hidden');
			getData(user.securityAcctId);
		}else{
			bar.box.querySelector('.data-box').classList.add('mui-hidden');
			bar.box.querySelector('.bind-bar').classList.remove('mui-hidden');
		} 
	}
	function loadStyles(){
		var styles =['medical/css.css'];
		createStyles(styles);
	}
	function formatData(data){
		try{
			data = data.content||'{}';
			return JSON.parse(data);
		}finally{
			return {};
		}
	}
	
	bar.name='medical';
	bar.setState = setState;
	bar.show = function(el,data){
		bar.box = el;
		bar.data = formatData(data);
		initUI();
	};
	//loadStyles();
	window.addEventListener('refresh_data',function(){
		bar.setState();
	},false);

	return bar;
}));
	