(function($,chart){
	var pieChart = function(conId){
		var pixelRatio = window.devicePixelRatio||1;
		var chartEle = g(conId),
			width = chartEle.offsetWidth * pixelRatio,
			height = chartEle.offsetHeight * pixelRatio;
			
		var canvas =  g('canvas');
		var ctx = canvas.getContext('2d');
		canvas.width = width ;
	    canvas.height = height;	
		
		var fontSize5 = 5*pixelRatio;
		var fontSize10 = 10*pixelRatio;
		var fontSize12 = 12*pixelRatio;
		var fontSize14 = 14*pixelRatio;
		var fontSize45 = 40*pixelRatio;
		
		var data,sectors,colors,space,caseSize,
			caseWidth,circleWidth,clockwise,speed,rate=9,
			textKeyName,valueKeyName,tipKeyName,selectIndex,
			showText=true;
		var cx = width/2,
			cy = height/2,
			radius = width/6,
			arc =  Math.PI/180;
		var animationId,progress,rendering;
		
		function drawBase(){
		 	//空格角度,每扇区弧度,每扇区格子数量,每格子弧度
		 	var spaceArc = space*arc, 
		 		sectionArc = 360 / sectors * arc,
				sectionCase =  (360 / sectors) / caseSize,
				caseArc = caseSize * arc;
			var outRadius = radius + (caseWidth+circleWidth)/2;
			
			for(var i=0;i<sectors;i++) {
				ctx.save();
				ctx.beginPath();
				var ssArc = i*sectionArc;
				var esArc = (i+1)*sectionArc;
			    ctx.arc(cx, cy, radius, ssArc + spaceArc, esArc - spaceArc);
			    ctx.lineWidth = circleWidth;
			    ctx.strokeStyle = colors[i];
			    ctx.stroke();
				ctx.closePath();
				ctx.restore();
				
				for(var m=0;m<sectionCase;m++){
					ctx.save();
					ctx.beginPath();
					var scArc = m*caseArc + ssArc ;
					var ecArc = (m+1)*caseArc + ssArc;
					ctx.arc(cx, cy, outRadius, scArc+ spaceArc, ecArc - spaceArc);
				   	ctx.lineWidth = caseWidth;
				    ctx.strokeStyle = 'rgba(255,255,255,0.3)';
				    ctx.stroke();
					ctx.closePath();
					ctx.restore();
				}
				var dots =  (360 / sectors) / 5,
					dotArc = 5 * arc,
					dotRadius = radius + (2*caseWidth);
				for(var m=0;m<dots;m++){
					ctx.save();
					ctx.beginPath();
					var scArc = m*dotArc + ssArc ;
					var ecArc = (m+1)*dotArc + ssArc;
					ctx.arc(cx, cy, dotRadius, scArc+ spaceArc, ecArc - spaceArc);
				   	ctx.lineWidth = i===selectIndex?pixelRatio*1.5:pixelRatio;
				    ctx.strokeStyle = i===selectIndex?colors[i]:'rgba(255,255,255,0.3)';
				    ctx.stroke();
					ctx.closePath();
					ctx.restore();
				}
			}
		}
		function drawText(){
			var dotRadius = radius + (2*caseWidth);
			for(var x=0;x<sectors;x++){
				ctx.save();
				var sectionArc = (2*Math.PI/sectors)*(x+1)+Math.PI/sectors;
				var sx = dotRadius*Math.sin(sectionArc)+width/2;
				var sy = height/2-dotRadius* Math.cos(sectionArc);
				var ey = height*0.14;
				var ex1,ex2,ex3;
				if(sy>cy) ey = height * 0.86
				if(sx>cx){ 
					ex1 = sx+15*pixelRatio;
					ex2 = sx+15*pixelRatio+20*pixelRatio;
					ex3 = ex2+5*pixelRatio;
					ctx.textAlign="start";
				}else{
					ex1 = sx-15*pixelRatio;
					ex2 = sx-15*pixelRatio-20*pixelRatio;
					ex3 = ex2-5*pixelRatio;
					ctx.textAlign="end";
				}
				ctx.beginPath();
				ctx.lineWidth = 1*pixelRatio;
				ctx.strokeStyle = 'rgba(255,255,255,0.8)';
				ctx.moveTo(sx,sy);
				ctx.lineTo(ex1,ey);
				ctx.lineTo(ex2,ey);
				ctx.stroke();
			    ctx.closePath();
			    
			    ctx.beginPath();
			    ctx.arc(ex2,ey,2*pixelRatio, 0, 2*Math.PI);
			    ctx.fillStyle='rgba(255,255,255,0.8)';
			    ctx.closePath();
				ctx.fill();
				
				ctx.font=fontSize12+'px sans-serif';
			   	ctx.fillStyle='rgba(255,255,255,1)';
			   	ctx.fillText(data[x][textKeyName],ex3,ey+fontSize12/2);
			   	ctx.restore();
			   	
				if(x==selectIndex){
					ctx.beginPath();
					ctx.moveTo(ex2,ey-2*pixelRatio);
					ctx.bezierCurveTo(
						ex2-35*pixelRatio,
						ey-32*pixelRatio,
						ex2+35*pixelRatio,
						ey-32*pixelRatio,
						ex2,
						ey-2*pixelRatio);
					ctx.fillStyle='rgba(255,255,255,0.8)';
					ctx.fill();
					ctx.closePath();
					
					ctx.textAlign='center';
					ctx.font=fontSize10 + 'px sans-serif';
			   		ctx.fillStyle=colors[x];
			   		ctx.fillText(data[x][valueKeyName],ex2,ey-fontSize10-2*pixelRatio);
				}
				
			   	
			}
		}
		function drawValue(progress){
			ctx.font= fontSize5+'px sans-serif';
		 	//空格角度,每扇区弧度,每扇区格子数量,每格子弧度
		 	var spaceArc = space*arc, 
		 		sectionArc = 360 / sectors * arc,
				sectionCase =  (360 / sectors) / caseSize,
				caseArc = caseSize * arc;
			var outRadius = radius + (caseWidth+circleWidth)/2;
			
			ctx.save();
			ctx.font=fontSize45+'px sans-serif';
			ctx.textAlign='center';
		   	ctx.fillStyle='rgba(255,255,255,0.8)';
		   	ctx.fillText((data[selectIndex][valueKeyName]*progress).toFixed(0),cx,cy);
		   	
		   	ctx.font=fontSize14+'px sans-serif';
			ctx.textAlign='center';
		   	ctx.fillText(data[selectIndex][tipKeyName],cx+fontSize14/2,cy+fontSize14+3*pixelRatio);
		    ctx.restore();
			   	
			for(var x=0;x<sectors;x++) {
			 	var value = data[x][valueKeyName]/100*progress;
				var num = Math.ceil(value*sectionCase);
				var num1 = Math.floor(value*sectionCase);
				for(var y=0;y<num;y++){
					ctx.save();
					ctx.beginPath();
					ctx.lineWidth = caseWidth;
					if(clockwise){
						var scArc = y*caseArc + spaceArc;
						var ecArc = y===num-1&&num>num1 ?  value*sectionArc:  (y+1)*caseArc - spaceArc;
						ctx.arc(cx, cy, outRadius, (x+1)*sectionArc-scArc, (x+1)*sectionArc- ecArc,true);
					}else{
						var scArc = y*caseArc + spaceArc;
						var ecArc = y===num-1&&num>num1 ? value*sectionArc: (y+1)*caseArc - spaceArc;
						ctx.arc(cx, cy, outRadius, scArc + x*sectionArc, ecArc + x*sectionArc);
					}
					ctx.strokeStyle = 'rgba(255,255,255,0.9)';
				    ctx.stroke();  
					ctx.closePath();
					ctx.restore();
				}
		 	}
		}
		function draw(progress){
			drawBase();
			if(showText) drawText();
			drawValue(progress);
		}
		
		function render() {
			if(!rendering) return;
			/*progress += speed;
			if(progress>1)  {
				progress=1;
				rendering = false;
			}*/
			progress = progress + (1-progress)/rate;
			if(1-progress<0.0000001)  {
				progress=1;
				rendering = false;
			}
			ctx.clearRect( 0, 0, width, height );
		 	draw(progress);
		 	animationId = requestAnimationFrame( render );
		}
   		
   		function setData(options){
   			data = options.data;
			sectors = options.sectors || 1; //扇区数
			colors = options.colors || ['#0e6291','#90c121','#f6b52d','#ea3629'];//扇区颜色和扇区数对应
			space = options.space ? (options.space/2) : 0; //空白角度大小
			caseSize = options.caseSize || 15; //每个格子的角度大小
			caseWidth = (options.caseWidth || chartEle.offsetHeight/8)*2; //格子的高度
			circleWidth = (options.circleWidth || chartEle.offsetHeight/50)*2; //中心圈的高度
			speed = options.speed || 0.09; //
			rate = options.rate || 9; //
			clockwise = options.clockwise || false; //逆时针
			textKeyName = options.textKeyName || 'text'; 
			valueKeyName = options.valueKeyName || 'value';
			tipKeyName = options.tipKeyName || 'tipTxt';
			showText = options.showText===false?false:true;
			selectIndex = 0;
   		};
   		
		this.init = function(options){
			setData(options);
			drawBase();
			setTimeout(function(){
				rendering = true;
				progress = 0;
				render();
			},500);
		};
		
		this.setData = setData;
		this.drawBase = drawBase;
		
		this.start = function(){
			cancelAnimationFrame(animationId);
			rendering = true;
			progress = 0;
			render();
		};
		this.stop = function(){
			rendering = false;
			cancelAnimationFrame(animationId);
		};
		this.slideTo = function(num){
			selectIndex = num;
			rendering = true;
			render();
		}; 
	};
	var createInsHtml = function(data){
		g('proGroup').innerHTML='';
		data.forEach(function(pro,i){
			var item = document.createElement('div')
			item.className='mui-slider-item'+(i===0?' mui-active':'');
			item.innerHTML=[
				'<div class="pro" url="',pro.insUrl,'" pname="',pro.insName,'" pid="',pro.insId,'" price="',pro.insAmount,'" type="',pro.name,'">',
					'<div class="pro-img">',
						'<img src="',pro.insImg,'">',
					'</div>',
					'<div class="padded">',
						'<div class="pro-name">',pro.insName,'</div>',
						'<p class="pro-intro">',pro.insIntro,'</p>',
						'<div class="pro-price">',
							'<b class="price">',pro.insAmount,'</b><span>元起</span>',
						'</div>',
					'</div>',
					'<div class="mask"></div>',
				'</div>',
				'<div class="detail">',pro.introDetail,'</div>'
			].join('');
			g('proGroup').appendChild(item);
		});
	};
 	var refresh = function(){
		if(app.checkLogin()){
			var user = app.getUser();
			joySvc.getExpenseIndex(user.securityAcctId).then(function(data){
				
				g('title').innerText = data[0].name;
				createInsHtml(data);
				
				setTimeout(function(){
					chart.data = data;
					chart.setData({
						data:data,
						textKeyName:'name',
						valueKeyName:'index',
						tipKeyName:'safeValue',
						sectors:4,
						space:2,
						caseSize:15,
						//caseWidth:12,
						//circleWidth:2,
						clockwise:true
					});
					chart.start();
				},50);
				
			}).catch(function(err){
				console.error(err.message,err);
			});
		}else{
			app.toLogin();
		}
	};
	var initViews = function(){
		var halfWidth = document.body.clientWidth/2;
		g('chart').style.height = halfWidth+'px';
		var canvas = g('canvas');
		canvas.style.width = document.body.clientWidth+'px';
		canvas.style.height = halfWidth+'px';
		
		if(imbar.isImmersedStatusbar){
			g('pieBg').style.paddingTop= imbar.navBarHeight + 'px';
		}
		
		chart = new pieChart('chart');
		setTimeout(function(){
			chart.setData({
				data:[{name:'',index:100,safeValue:'加载中...'}],
				textKeyName:'name',
				valueKeyName:'index',
				tipKeyName:'safeValue',
				showText:false,
				sectors:1,
				space:2,
				caseSize:15,
				//caseWidth:12,
				//circleWidth:2,
				clockwise:true
			});
			chart.drawBase();
			chart.start();
		},10);
	};
	
	var itemWidth = g('proGroup').children[0].offsetWidth;
	var slider = $('#plist').slider();
	var scaleDivInt;
	var scaleDiv = function(event){
		var next = 0;
		if(slider.direction === 'left'){
			next = slider.slideNumber+1;
			if(slider.slideNumber===slider.itemLength-1) return;
		}else{
			next = slider.slideNumber-1;
			if(slider.slideNumber===0) return;
		}
		var scleft = Math.abs(event.detail.lastX);
		var leftin = Math.abs(scleft - slider.slideNumber*itemWidth)/itemWidth;
		leftin=leftin>1?1:leftin;
		
		var scc = 1-leftin*0.1,scc=scc<0.9?0.9:scc,
			scn = 0.9+leftin*0.1,scn=scn>1?1:scn,
			oc = leftin*0.6,on=0.6-leftin*0.6,on=on<0?0:on;
		g('proGroup').children[slider.slideNumber].style.transform='scale('+scc+')';
		g('proGroup').children[slider.slideNumber].style.webkitTransform='scale('+scc+')';
		g('proGroup').children[next].style.transform='scale('+scn+')';
		g('proGroup').children[next].style.webkitTransform='scale('+scn+')';
		g('proGroup').children[slider.slideNumber].querySelector('.mask').style.opacity= oc;
		g('proGroup').children[next].querySelector('.mask').style.opacity=on;
	};
	
	g('plist').addEventListener('scroll',function(event) {
		//clearTimeout(scaleDivInt);
		scaleDivInt = setTimeout(function(){
			scaleDiv(event);
		},1);
	},false);
	g('plist').addEventListener('slide',function(event) {
		var bgcls = ['green1','red','blue','green'];
		g('pieBg').className='pie '+bgcls[event.detail.slideNumber];
		g('proGroup').children[event.detail.slideNumber].classList.add('mui-active');
		g('title').innerText = chart.data[event.detail.slideNumber].name;
		chart.slideTo(event.detail.slideNumber);
	},false);
	$('#plist').on('tap','.pro',function(event) {
		var url = this.getAttribute('url');
		app.openUrl(url);
	},false);
	
	initViews();
	
	$.plusReady(function(){
		refresh();
	});

	window.addEventListener('refresh_data',function(){refresh()},false);
	
	// 埋点
	function joyTrack(){
		jTrack.regEventsLive('#proGroup','.pro','joySpreadInsClick','jtrack_ins_pid:attr.pid,jtrack_ins_pname:attr.pname,jtrack_ins_tname:attr.type,jtrack_ins_price:attr.price');
	}
	joyTrack();

})(mui,window.chart={});