(function (factory) {
    if (typeof exports === 'object') {
        // Node/CommonJS
        module.exports = factory();
    } else if (typeof define === 'function' && define.amd) {
        // AMD 
        define(factory);
    } else {
        // Browser globals (with support for web workers)
        var glob;
        try {
            glob = window;
        } catch (e) {
            glob = self;
        }
        glob.medical = factory();
    }
}(function(){
 	var bar = {};
	
	/** 设置页面显示内容
	 * @param {Object} data 获取的数据
	 * @param {Object} curIndex  当前模块的索引
	 */
	function setData(data, curIndex){
		var curParent = bar[curIndex];
		if (!data.expenseMinMz && !data.expenseMaxMz) {
			curParent.querySelector('.mzbx').classList.add('mui-hidden');
		} else {
			var oDown = curParent.querySelector('.reimburse-down .num');
			var oUp = curParent.querySelector('.reimburse-up .num');
			setInnerHtml(oDown, oUp, data.expenseMinMz, data.expenseMaxMz);
		}
		if (!data.expensePolicyMz && !data.expenseEstimateMz) {
			curParent.querySelector('.zybx').classList.add('mui-hidden');
		} else {
			var oDown = curParent.querySelector('.ratio-down .num');
			var oUp = curParent.querySelector('.ratio-up .num');
			setInnerHtml(oDown, oUp, data.expensePolicyMz, data.expenseEstimateMz);
		}
	}
	/**
	 * @param {Object} oDown 最小限额
	 * @param {Object} oUp   最大限额
	 * @param {String} donwData  填充的数据 
	 * @param {String} upData 数据key
	 */
	function setInnerHtml(oDown, oUp, downData, upData){
		var result = splitStr(downData);
		oDown.innerHTML = '<em>' + (result.first || (!result.second ? '--' : '')) + '</em>' + (result.second ? result.second : '');
		result = splitStr(upData);
		oUp.innerHTML = '<em>' + (result.first || (!result.second ? '--' : '')) + '</em>' + (result.second ? result.second : '');
	}
	/**
	 * 拆解 金额 
	 * @param {String} str 
	 */
	function splitStr(str){
		var numRE = /^[\d]+/g;
		var match;
		var result = {};
		if (match = numRE.exec(str)) {
			result.first = match[0];
			result.second = str.replace(numRE, '');
		} else {
			result.second = str;
		}
		return result;
	}
 	function getData(curIndex){
		// 报销类型 （app3.0.1版本报销需要区分门诊、住院）
		var expenseType = bar['data_' + curIndex].expenseType;
		// 门诊和住院需要重新设置 缓存key (缓存key值需要在indexdb.js文件进行配置，暂时不做配置了，如果后续发现加载缓慢等问题，在做配置)
 		var storeName = 'reimburse_' + expenseType;
 		jdb.getData(storeName,config.cityCode).then(function(data){
 			if (!data) return;
			setData(data, curIndex);
		}).catch(function(err){
			console.error(err.message,err);
		});
 		setTimeout(function(){
			joySvc.getMedicExpense(config.cityCode,config.perType, expenseType).then(function(data){
				data = transFormData(data);
				setData(data, curIndex);
				data.cityType = config.cityCode+config.perType;
				jdb.saveData(storeName,data,'cityType');
			}).catch(function(err){
				console.error(err.message,err);
			});
		},50);
 	}
	// 门诊和医院返回的数据格式不一样需要序列化（因为门诊和住院使用的是同一个模板）
	function transFormData(data){
		for (var i in data) {
			data[i] = !data[i] ? '' : data[i].trim();
		}
		if ('expenseMinZy' in data) {
			data.expenseMinMz = data.expenseMinZy;
			data.expenseEstimateMz = data.expenseEstimateZy;
			data.expenseMaxMz = data.expenseMaxZy;
			data.expensePolicyMz = data.expensePolicyZy;
		}
		return data;
	}
	/**
	 * @param {Object} curIndex 当前模块索引
	 */
	function initUI(curIndex){
		var templateUrl = 'reimburse-2/tpl.html';
		mui.ajax(templateUrl,{
			dataType:'html',
			async:true,           
			success:function(html){
				bar[curIndex].innerHTML = html;
				mui(bar[curIndex]).on('tap','.lkbox a',function(){
					mui.openWindow(this.getAttribute('href'));
				});
			    getData(curIndex);
			}
		});
	}

	function loadStyles(){
		var styles =['reimburse/css.css'];
		createStyles(styles);
	}
	function formatData(data){
		var result = {};
		try{
			result = data.content||'{}';
			result = JSON.parse(result);
		} catch (e) {
			console.log(e);
		} finally{
			return result;
		}
	}
	var num = 1;
	bar.name='reimburse';
	bar.show = function(el,data){
		//注意： 为了区分门诊和住院公用这个模块，需要把当前传入的dom区分保存
		bar[num] = el;
		bar['data_' + num] = formatData(data);
		initUI(num);
		num ++;
	};
	return bar;
}));
	