(function (factory) {
    if (typeof exports === 'object') {
        // Node/CommonJS
        module.exports = factory();
    } else if (typeof define === 'function' && define.amd) {
        // AMD 
        define(factory);
    } else {
        // Browser globals (with support for web workers)
        var glob;
        try {
            glob = window;
        } catch (e) {
            glob = self;
        }
        glob.statebar = factory();
    }
}(function(){
	var bar = {};
	/**
	 * @description 获取参保险种缴费状态
	 * @param {Number} sbkid 社保信息Id
	 * @param {String} type 缴费险种分类 01:养老,02:失业,03:医疗,04:工伤,05:生育
	 */
	function getInsPayMonths(sbkid,type) {
		var storeName = 'InsPayedMonths';
		var insPayedMonths = {id:sbkid + "_" + type,ins:{}};
		var callback = function(data){
			var payedMonths = data[type],
				totalMonths = data.totlMonths||0;
			
			//targetMonths = bar.data.totalMonths,
			//per = parseInt(payedMonths/targetMonths*100);
				
			bar.monthPan.innerHTML= '<em>' + payedMonths + '</em>个月';
			bar.monthTotal.innerHTML='<em>' + totalMonths+'</em>个月';
			
			//bar.monthPan.innerHTML=payedMonths+'个月';
			//bar.targetMonths.innerHTML='需缴纳'+targetMonths+'个月';
			//bar.oJinduBar.style.width = per>100 ? '100%':per+'%';
		};
		jdb.getData(storeName,sbkid).then(function(res){
			if(res) {
				mui.extend(true,insPayedMonths,res);
				callback(res.ins);
			}
		}).catch(function(err){
			console.log(err);
		});
		setTimeout(function(){
			joySvc.getSecConPayMonths(sbkid,type).then(function(data){
				insPayedMonths.ins[type] = data;
				callback(insPayedMonths.ins);
				setTimeout(function(){
					jdb.saveData(storeName,insPayedMonths,'id');
				},50);
			}).catch(function(err){
				console.error(err.message,err);
			});
		},50);
	};
	
	/**
	 * @description 获取参保险种缴费状态
	 * @param {Object} sbkid 社保信息Id
	 * @param {String} type 缴费险种分类 01:养老,02:失业,03:医疗,04:工伤,05:生育
	 */
	function getInsState(sbkid,type) {
		var storeName = 'InsPayState';
		var callback = function(data){
			//bar.zhuangtaiPan.innerHTML=data[type]||'－－';
			bar.stateTxt.innerHTML=data[type]||'－－';
		};
		jdb.getData(storeName,sbkid).then(function(res){
			if(res) callback(res.ins);
		}).catch(function(err){
			console.log(err);
		});
		setTimeout(function(){
			joySvc.getInsState(sbkid).then(function(data){
				callback(data);
				jdb.saveData(storeName,{id:sbkid,ins:data},'id');
			}).catch(function(err){
				console.error(err.message,err);
			});
		},50);
	};

	function getInsIAInfo(sbkid,type){
		var storeName = 'InsIAInfo';
		var dataid = sbkid+'_'+type;
		var setInfo = function(data){
			bar.stateTxt.innerHTML=data.status||'－－';
			if(data.status=='异常'){
				bar.stateTxt.classList.add('warn');
			}
			bar.monthPan.innerHTML=data.continuousMonths? '<em>' + data.continuousMonths+'</em>个月':'－－';
			bar.monthTotal.innerHTML=data.totalMonths? '<em>' + data.totalMonths+'</em>个月':'－－';
		};
		
		jdb.getData(storeName,dataid).then(function(res){
			if(res) setInfo(res.ins);
		}).catch(function(err){
			console.log(err);
		});
		setTimeout(function(){
			joySvc.getInsIAInfo(sbkid,type).then(function(data){
				if(data) setInfo(data);
				jdb.saveData(storeName,{id:dataid,ins:data},'id');
			}).catch(function(err){
				console.error(err.message,err);
			});
		},50);
	}

	function initUI(){
		var templateUrl = 'statebar/tpl.html';
		mui.ajax(templateUrl,{
			dataType:'html',
			async:true,           
			success:function(html){
				bar.box.innerHTML = html;
				bar.monthPan = bar.box.querySelector('.conti-month');
				bar.monthTotal = bar.box.querySelector('.total-month');
				bar.stateTxt = bar.box.querySelector('.state-txt');
				bar.stateBox = bar.box.querySelector('.state-info');
				bar.bindBox = bar.box.querySelector('.state-bind');
				bar.bindBtn = bar.box.querySelector('.tip-btn');
				
				//bar.oJinduBar = bar.box.querySelector('#jindu_top');
				//bar.jindutiao = bar.box.querySelector('#jindutiao');
				//bar.targetMonths = bar.box.querySelector('#target_months');
				
				setState();
				bar.bindBtn.addEventListener('tap', function() {
					if(config.user){
						app.showSheBaoAdd({
							viewId:mui.currentWebview.id,
							events:['refresh_data']
						});
					}else{
						app.toLogin();
					}
				}, false);
			}
		});
	}
	
	function setState(){
		var data = bar.data;
		var user = config.user;
		if(user && user.securityAcctId) {
			bar.sbkid = user.securityAcctId;
			bar.stateBox.classList.remove('mui-hidden');
			bar.bindBox.classList.add('mui-hidden');
			getInsIAInfo(bar.sbkid,data.type);
			
			//getInsState(bar.sbkid,data.type);
			//getInsPayMonths(bar.sbkid,data.type);
			
			//bar.jindutiao.classList[data.pgbar!=false?'remove':'add']('mui-hidden');
			//bar.targetMonths.classList[data.months!=false?'remove':'add']('mui-hidden');
		} else {
			bar.stateBox.classList.add('mui-hidden');
			bar.bindBox.classList.remove('mui-hidden');
			
			bar.stateTxt.innerHTML = '－－';	
			bar.monthPan.innerHTML = '－－';	
			bar.monthTotal.innerHTML = '－－';
			
			//bar.zhuangtaiPan.innerHTML = '－－';	
			//bar.jindutiao.classList.add('mui-hidden');
		}
	}
	function loadStyles(){
		var styles =['statebar/css.css'];
		createStyles(styles);
	}
	function formatData(data){
		try{
			data = data.content||'{}';
			return JSON.parse(data);
		}catch(e){
			return {type:'01', pgbar:true, months:true}
		}
	}
	
	
	bar.name='statebar';
	bar.setState = setState;
	bar.show = function(el,data){
		bar.box = el;
		bar.data = formatData(data);
		initUI();
	};
	//loadStyles();
	window.addEventListener('refresh_data',function(){
		console.log(config);
		bar.setState();
	},false);
	return bar;
}));
